@@ -1,9 +1,7 @@
-use crate::error::*;
-use crate::traits::*;
-use crate::{Any, Class, Header, Length, Tag};
-use std::borrow::Cow;
-use std::convert::TryFrom;
-use std::convert::TryInto;
+use crate::*;
+use alloc::borrow::Cow;
+use alloc::vec;
+use core::convert::{TryFrom, TryInto};
 
 #[cfg(feature = "bigint")]
 #[cfg_attr(docsrs, doc(cfg(feature = "bigint")))]
@@ -14,10 +12,7 @@ pub use num_bigint::{BigInt, BigUint, Sign};
 ///
 /// Returns a byte array of the requested size containing a big endian integer.
 fn decode_slice(any: Any<'_>) -> Result<&[u8]> {
-    let bytes = match any.data {
-        Cow::Borrowed(b) => b,
-        Cow::Owned(_) => unreachable!(),
-    };
+    let bytes = any.data;
 
     // The `INTEGER` type always encodes a signed value, so for unsigned
     // values the leading `0x00` byte may need to be removed.
@@ -40,6 +35,9 @@ fn decode_slice(any: Any<'_>) -> Result<&[u8]> {
 /// Decode an unsigned integer into a byte array of the requested size
 /// containing a big endian integer.
 fn decode_array_uint<const N: usize>(any: Any<'_>) -> Result<[u8; N]> {
+    if is_highest_bit_set(&any.data) {
+        return Err(Error::IntegerNegative);
+    }
     let input = decode_slice(any)?;
 
     if input.len() > N {
@@ -113,6 +111,7 @@ macro_rules! impl_int {
             const TAG: Tag = Tag::Integer;
         }
 
+        #[cfg(feature = "std")]
         impl ToDer for $int {
             fn to_der_len(&self) -> Result<usize> {
                 let int = Integer::from(*self);
@@ -169,6 +168,7 @@ macro_rules! impl_uint {
             const TAG: Tag = Tag::Integer;
         }
 
+        #[cfg(feature = "std")]
         impl ToDer for $ty {
             fn to_der_len(&self) -> Result<usize> {
                 let int = Integer::from(*self);
@@ -283,14 +283,18 @@ impl<'a> Integer<'a> {
     #[cfg(feature = "bigint")]
     #[cfg_attr(docsrs, doc(cfg(feature = "bigint")))]
     pub fn as_bigint(&self) -> BigInt {
-        BigInt::from_bytes_be(Sign::Plus, &self.data)
+        BigInt::from_signed_bytes_be(&self.data)
     }
 
     /// Returns a `BigUint` built from this `Integer` value.
     #[cfg(feature = "bigint")]
     #[cfg_attr(docsrs, doc(cfg(feature = "bigint")))]
-    pub fn as_biguint(&self) -> BigUint {
-        BigUint::from_bytes_be(&self.data)
+    pub fn as_biguint(&self) -> Result<BigUint> {
+        if is_highest_bit_set(&self.data) {
+            Err(Error::IntegerNegative)
+        } else {
+            Ok(BigUint::from_bytes_be(&self.data))
+        }
     }
 
     /// Build an `Integer` from a constant array of bytes representation of an integer.
@@ -417,7 +421,7 @@ impl<'a> TryFrom<Any<'a>> for Integer<'a> {
     fn try_from(any: Any<'a>) -> Result<Integer<'a>> {
         any.tag().assert_eq(Self::TAG)?;
         Ok(Integer {
-            data: any.into_cow(),
+            data: Cow::Borrowed(any.data),
         })
     }
 }
@@ -433,6 +437,7 @@ impl<'a> Tagged for Integer<'a> {
     const TAG: Tag = Tag::Integer;
 }
 
+#[cfg(feature = "std")]
 impl ToDer for Integer<'_> {
     fn to_der_len(&self) -> Result<usize> {
         let sz = self.data.len();
